@@ -1,5 +1,5 @@
 package App::Netdisco::DB::ResultSet::Device;
-use base 'DBIx::Class::ResultSet';
+use base 'App::Netdisco::DB::ResultSet';
 
 use strict;
 use warnings FATAL => 'all';
@@ -204,6 +204,23 @@ sub search_by_field {
         || NetAddr::IP::Lite->new('255.255.255.255') );
     }
 
+    # For Search on Layers
+    my @layer_search = ( '_', '_', '_', '_', '_', '_', '_' );
+    # @layer_search is computer indexed, left->right
+    my $layers = $p->{layers};
+    if ( defined $layers && ref $layers ) {
+      foreach my $layer (@$layers) {
+        next unless defined $layer and length($layer);
+        next if ( $layer < 1 || $layer > 7 );
+        $layer_search[ $layer - 1 ] = 1;
+      }
+    }
+    elsif ( defined $layers ) {
+      $layer_search[ $layers - 1 ] = 1;
+    }
+    # the database field is in order 87654321
+    my $layer_string = join( '', reverse @layer_search );
+
     return $rs
       ->search_rs({}, $attrs)
       ->search({
@@ -214,6 +231,8 @@ sub search_by_field {
             { '-ilike' => "\%$p->{location}\%" }) : ()),
           ($p->{description} ? ('me.description' =>
             { '-ilike' => "\%$p->{description}\%" }) : ()),
+          ($p->{layers} ? ('me.layers' =>
+            { '-ilike' => "\%$layer_string" }) : ()),
 
           ($p->{model} ? ('me.model' =>
             { '-in' => $p->{model} }) : ()),
@@ -352,8 +371,9 @@ sub carrying_vlan {
     die "vlan number required for carrying_vlan\n"
       if ref {} ne ref $cond or !exists $cond->{vlan};
 
-    $cond->{'vlans.vlan'} = $cond->{vlan};
-    $cond->{'port_vlans.vlan'} = delete $cond->{vlan};
+    $cond->{'-and'} ||= [];
+    push @{$cond->{'-and'}}, 'vlans.vlan' => $cond->{vlan};
+    push @{$cond->{'-and'}}, 'port_vlans.vlan' => delete $cond->{vlan};
 
     return $rs
       ->search_rs($cond,
@@ -402,7 +422,7 @@ sub carrying_vlan_name {
     die "vlan name required for carrying_vlan_name\n"
       if ref {} ne ref $cond or !exists $cond->{name};
 
-    $cond->{'vlans.description'} = { '-ilike' => $cond->{name} };
+    $cond->{'vlans.description'} = { '-ilike' => delete $cond->{name} };
 
     return $rs
       ->search_rs({}, {
@@ -413,6 +433,31 @@ sub carrying_vlan_name {
       ->search($cond, $attrs);
 }
 
+=head2 has_layer( $layer )
+
+ my $rset = $rs->has_layer(3);
+
+This predefined C<search()> returns a ResultSet of matching rows from the
+Device table of devices advertising support of the supplied layer in the
+OSI Model.
+
+=over 4
+
+=item *
+
+The C<layer> parameter must be an integer between 1 and 7.
+
+=cut
+
+sub has_layer {
+    my ( $rs, $layer ) = @_;
+
+    die "layer required and must be between 1 and 7\n"
+        if !$layer || $layer < 1 || $layer > 7;
+
+    return $rs->search_rs( \[ 'substring(layers,9-?, 1)::int = 1', $layer ] );
+}
+
 =head2 get_models
 
 Returns a sorted list of Device models with the following columns only:
@@ -438,7 +483,7 @@ sub get_models {
     select => [ 'vendor', 'model', { count => 'ip' } ],
     as => [qw/vendor model count/],
     group_by => [qw/vendor model/],
-    order_by => [{-asc => 'vendor'}, {-desc => 'count'}, {-asc => 'model'}],
+    order_by => [{-asc => 'vendor'}, {-asc => 'model'}],
   })
 
 }
@@ -468,32 +513,11 @@ sub get_releases {
     select => [ 'os', 'os_ver', { count => 'ip' } ],
     as => [qw/os os_ver count/],
     group_by => [qw/os os_ver/],
-    order_by => [{-asc => 'os'}, {-desc => 'count'}, {-asc => 'os_ver'}],
+    order_by => [{-asc => 'os'}, {-asc => 'os_ver'}],
   })
 
 }
 
-=head2 get_distinct_col( $column )
-
-Returns an asciibetical sorted list of the distinct values in the given column
-of the Device table. This is useful for web forms when you want to provide a
-drop-down list of possible options.
-
-=cut
-
-sub get_distinct_col {
-  my ($rs, $col) = @_;
-  return $rs unless $col;
-
-  return $rs->search({},
-    {
-      columns => [$col],
-      order_by => $col,
-      distinct => 1
-    }
-  )->get_column($col)->all;
-}
-
 =head2 with_port_count
 
 This is a modifier for any C<search()> which
@@ -544,7 +568,6 @@ sub delete {
   my $devices = $self->search(undef, { columns => 'ip' });
 
   foreach my $set (qw/
-    Community
     DeviceIp
     DeviceVlan
     DevicePower
@@ -555,6 +578,15 @@ sub delete {
       )->delete;
   }
 
+  $schema->resultset('Community')->search({
+    ip => { '-in' => $devices->as_query },
+    snmp_auth_tag => undef,
+  })->delete;
+
+  $schema->resultset('Community')->search(
+    { ip => { '-in' => $devices->as_query } },
+  )->update({snmp_comm_rw => undef});
+
   $schema->resultset('Admin')->search({
     device => { '-in' => $devices->as_query },
     action => { '-like' => 'queued%' },
